
varying vec3 normal, lightDir;
varying vec4 vpos; // position of this vertex in MODEL coordinates (i.e.,  before rotation, etc.)
varying vec3 nxtan; // vector tangent to sphere at this vertex and pointing EAST along line of latitude
varying vec3 nytan; // vector tangent to sphere at this vertex and pointing NORTH along line of longitude

void main()
{
  vec3 xtan, ytan;   // nxtan, nytan, but in MODEL coordinates

  // standard stuff for diffuse lighting, copied from Phong example

  normal = normalize(gl_NormalMatrix * gl_Normal);
  lightDir = normalize(vec3(gl_LightSource[0].position - gl_ModelViewMatrix * gl_Vertex));

  // we are assuming sphere is centered at (0, 0, 0) (SHOULD pass with uniforms) 

  // this and xtan and ytan are in MODEL coordinates -- they will rotate as sphere rotates, for example.
  // we can treat vpos like the (non-unit) normal here because of our assumption about sphere center
  vpos = gl_Vertex;

  // 90-degree counterclockwise rotation of XZ component of vector from center of sphere to this vertex.
  // because sphere is rotated 90 degrees around X axis, we actually need to treat Y like Z and vice versa
  // for this to work.  also have to flip because -Y is into screen instead of out
  xtan = normalize(vec3(vpos.y, -vpos.x, 0.0));

  // cross product of normal and xtan makes ytan under right-hand rule (this is like the bitangent...)
  ytan = normalize(cross(vpos.xyz, xtan));  

  // now change to eye coordinates, because that's what normal and lightDir are in

  nxtan = normalize(gl_NormalMatrix * xtan);
  nytan = normalize(gl_NormalMatrix * ytan);

  // finish up

  gl_TexCoord[0] = gl_MultiTexCoord0;
  gl_Position = ftransform();
} 
